#!/usr/bin/env python3
"""Script to locally build a docker image
"""
import argparse
import datetime
import json
import logging
import os
import signal
import sys
from subprocess import call, check_output
from typing import List, Optional

from .config import ODOO_7, ODOO_8, ODOO_10, ODOO_11, ODOO_13, ODOO_14, ODOO_15, Config
from .docker_build_clean import clean
from .docker_build_copy import copy
from .docker_client import DockerClient
from .parsing import apply, basic_parser

_logger = logging.getLogger(__name__)

__version__ = "1.4.0"
__date__ = "2018-04-04"
__updated__ = "2023-03-02"


def add_build_options(parser: argparse.ArgumentParser):
    """Add build options to parser"""
    parser.add_argument(
        "--build-arg",
        help="build arg for the image, formatted like FOO=BAR [default: %(default)s]",
        default=None,
        nargs="*",
    )


def get_build_options(namespace: argparse.Namespace) -> List[str]:
    """Return build options usable in :func:`build_local_image`."""
    result = []
    if namespace.build_arg:
        for arg in namespace.build_arg:
            result.append("--build-arg")
            result.append(arg)
    return result


def __parser() -> argparse.ArgumentParser:
    program_version_message = f"%(prog)s {__version__} ({__updated__})"
    program_short_description = __doc__.split(".", maxsplit=1)[0]
    program_license = f"""{program_short_description}

      Created by Vincent Hatakeyama on {__date__}.
      Copyright 2018-2023 XCG Consulting. All rights reserved.

      Licensed under the MIT License

      Distributed on an "AS IS" basis without warranties
      or conditions of any kind, either express or implied.

    USAGE
    """
    parser = basic_parser(program_license, program_version_message)
    return parser


def main(argv: Optional[List[str]] = None) -> int:
    """Parse arguments and docker build"""
    # TODO the script assume it is launched from the parent super project
    project_path = os.path.realpath(".")
    project_name = os.path.basename(project_path)
    # TODO changez pour une vérification de la présence d’un Dockerfile
    if project_name == "odoo_scripts":
        _logger.fatal("You must run this script from the super project")
        return 1

    # TODO add a way to store configuration options in a project file
    # Argument parsing
    parser = __parser()
    add_build_options(parser)
    # TODO add tag option, and maybe force tag option
    parser.add_argument(
        "--ensureconf",
        help="ensureconf [default: %(default)s]",
        action="store_true",
    )
    parser.add_argument(
        "--push", help="Push image [default: %(default)s]", action="store_true"
    )
    parser.add_argument(
        "--dev",
        help="add dev feature to generated image [default: %(default)s]",
        action="store_true",
    )
    parser.add_argument(
        "--no-pull",
        help="indicate to docker to not pull the base image [default: %(default)s]",
        action="store_true",
    )
    parser.add_argument(
        "--local",
        help="Use local image name",
        action="store_true",
        dest="local",
        default=None,
    )
    # TODO (maybe) add argument for other build arg

    nmspc = parser.parse_args(argv)
    apply(nmspc)
    ensureconf = nmspc.ensureconf
    dev = nmspc.dev
    push = nmspc.push

    dockerfile = "Dockerfile"
    if not os.path.exists(dockerfile):
        _logger.critical("No dockerfile found")
        return 1

    c = Config()
    odoo_type = c.odoo_type
    repository = c.local_image_name if nmspc.local else c.repository
    _logger.debug("Docker repository: %s", repository)
    # TODO ensureconf
    if ensureconf:
        raise NotImplementedError
    # call build copy
    copy()

    # clean on exit
    def signal_handler(code, frame):
        # TODO handle the case when odoo_scripts is not installed
        clean()
        # XXX needed?

    signal.signal(signal.SIGINT, signal_handler)
    signal.signal(signal.SIGTERM, signal_handler)

    # used to retag
    tags: List[str] = []

    buildargs = {}
    if os.path.exists(".hg"):
        description = check_output(
            [
                "hg",
                "log",
                "-r",
                ".",
                "-T",
                "{latesttag}{sub('^-0-.*', '', '-{latesttagdistance}-h{node|short}')}",
            ]
        ).decode()
        buildargs["VERSION"] = description
        buildargs["SENTRY_RELEASE_ARG"] = description
        tags_string = check_output(["hg", "identify", "--tags"]).decode()
        if tags_string:
            tags = tags_string.split()
        buildargs["VCS_URL"] = check_output(["hg", "paths", "default"]).decode()
        buildargs["VCS_REF"] = check_output(["hg", "identify", "--id"]).decode()
    elif os.path.exists(".git"):
        description = check_output(["git", "describe"]).decode()
        buildargs["VERSION"] = description
        buildargs["SENTRY_RELEASE_ARG"] = description
        tags_string = check_output(["git", "describe", "--tags"]).decode()
        if tags_string:
            tags = tags_string.split()
    buildargs["BUILD_DATE"] = datetime.datetime.now().isoformat()
    if nmspc.build_arg:
        for arg in nmspc.build_arg:
            a = arg.split("=")
            buildargs[a[0]] = a[1]
    _logger.debug("Build args: %s", buildargs)
    if dev:
        debug_dockerfile = "Dockerfile.debug"
        call(["cp", dockerfile, debug_dockerfile])

        with open(debug_dockerfile, "a") as myfile:
            myfile.write("\n# Developer helpers\n")
            if odoo_type in (
                ODOO_7,
                ODOO_8,
                ODOO_10,
                ODOO_11,
                ODOO_13,
                ODOO_14,
                ODOO_15,
            ):
                myfile.write("USER root\nRUN apt-get update -qq\n")
                myfile.write(
                    "RUN DEBIAN_FRONTEND=noninteractive apt-get install -y -qq "
                )
            if odoo_type in (ODOO_11, ODOO_13, ODOO_14, ODOO_15):
                myfile.write(
                    "python3-watchdog python3-ipdb python3-pyinotify python3-wheel\n"
                )
                # newer setuptools_scm>6 is not compatible with python 3.5 due
                # to the use of f-strings
                myfile.write(
                    "RUN python3 -m pip install "
                    "\"setuptools_scm<6;python_version<='3.5'\"\n"
                )
                # Install pdb++
                myfile.write("RUN python3 -m pip install pdbpp\n")
                myfile.write("USER odoo\n")
            elif odoo_type in (ODOO_10,):
                myfile.write("python-ipdb\n")
                myfile.write(
                    "RUN pip install watchdog --disable-pip-version-check "
                    "--system --no-cache-dir --only-binary wheel"
                )
            elif odoo_type in (ODOO_7, ODOO_8):
                myfile.write("python-ipdb\n")
                myfile.write("RUN pip install pyinotify")
            else:
                myfile.write(
                    "RUN python3 -m pip --no-cache-dir install --user pdbpp watchdog "
                    "ipdb pyinotify\n"
                )

        dockerfile = debug_dockerfile
        # TODO remove temp image

    docker_client = DockerClient.client
    pull = not nmspc.no_pull
    _logger.debug("Docker Pull %s", pull)
    builder = docker_client.api.build(
        path=".",
        rm=True,
        pull=pull,
        buildargs=buildargs,
        tag=repository,
        dockerfile=dockerfile,
    )
    for line in builder:
        d = json.loads(line.decode("utf-8"))
        if "stream" in d:
            for line in d["stream"].rstrip().split("\n"):
                _logger.info(line)
        if "errorDetail" in d:
            _logger.fatal(d["errorDetail"])
            return 1
    if dev:
        call(["rm", dockerfile])
    # retag
    for tag in tags:
        if tag != "tip":
            _logger.info("Docker with tag %s", tag)
            docker_client.images.get(repository).tag(repository, tag=tag)

    if push:
        _logger.info("Docker push %s", repository)
        for line in docker_client.images.push(repository, stream=True, decode=True):
            _logger.debug(line)
        for tag in tags:
            _logger.info("Docker push %s:%s", repository, tag)
            for line in docker_client.images.push(
                repository, tag=tag, stream=True, decode=True
            ):
                _logger.debug(line)
    # XXX call cleanup more intelligently
    signal_handler(0, None)
    return 0


def build_local_image(build_options: List[str], force: bool = False) -> int:
    """Build the local image if necessary
    :param force: force build if True
    :return: 0 if success, docker failed error code otherwise
    """
    c = Config()
    image = c.local_image_name

    image_list = DockerClient.client.images.list(name=image)
    if not image_list:
        _logger.info("Image %s does not exist", image)
    else:
        _logger.info("Image %s exists", image)
    if not image_list or force:
        _logger.info("Building image %s", image)
        arguments = ["--dev", "--local"] + build_options
        failed = main(arguments)
        if failed:
            return failed
    return 0


if __name__ == "__main__":
    sys.exit(main())
