#!/usr/bin/env python3
"""Tools to help build an image
"""
import logging
import os
import sys
from subprocess import check_call
from typing import List, Optional

from .config import Config
from .list_modules import list_modules
from .parsing import apply, basic_parser

_logger = logging.getLogger(__name__)

__version__ = "2.2.0"
__date__ = "2020-06-30"
__updated__ = "2023-03-31"

PYTHON_PACKAGES_DIR = "python_packages"


def __parser():
    program_version_message = f"%(prog)s {__version__} ({__updated__})"
    program_shortdesc = __doc__.split(".", maxsplit=1)[0]
    program_license = f"""{program_shortdesc}

      Created by Vincent Hatakeyama on {__date__}.
      Copyright 2020, 2021, 2023 XCG Consulting. All rights reserved.

      Licensed under the MIT License

      Distributed on an "AS IS" basis without warranties
      or conditions of any kind, either express or implied.

    USAGE
    """
    parser = basic_parser(program_license, program_version_message)
    return parser


def main(argv: Optional[List[str]] = None) -> int:
    """Copy modules for a build, callable version that parses arguments"""
    parser = __parser()
    nmspc = parser.parse_args(argv)
    apply(nmspc)
    copy()
    return 0


def _target_path(path: str) -> str:
    return path.replace(os.path.sep, "-")


def copy(build_package_locally: Optional[bool] = None):
    """Copy modules for a build

    :param build_package_locally: build package that have package compile set to True
      rather than just copy them. (in CI, build package to reduce artifact size, locally
      default to copying them)
    """
    c = Config()
    modules = [os.path.realpath(module) for module in c.modules]
    target = "odoo_modules"
    if not os.path.exists(target):
        os.mkdir(target)
    cmd = (
        [
            "rsync",
            "--delete",
            "--cvs-exclude",
            "--include=core",
            "--exclude=.hg*",
            "--exclude=.git*",
            "--links",
            "--exclude='*.pyc'",
            "-r",
            "--times",
        ]
        + modules
        + [target]
    )
    _logger.debug(" ".join(cmd))
    check_call(cmd)

    target = PYTHON_PACKAGES_DIR
    if not os.path.exists(PYTHON_PACKAGES_DIR):
        os.mkdir(target)
    copy_paths = set()
    packages_to_compile = []
    requirements = []
    ci_mode = "CI" in os.environ and os.environ["CI"].lower() == "true"
    # Try to include the CVS information in target so that setuptools-scm and
    # setuptools-odoo work as expected
    for package, options in c.python_packages.items():
        if options["compile"] in ("True", "true") and (
            ci_mode or build_package_locally
        ):
            packages_to_compile.append(package)
        else:
            cvs_parent = package
            package_path_in_target = ""
            while not (
                os.path.exists(os.path.join(cvs_parent, ".git"))
                or os.path.exists(os.path.join(cvs_parent, ".hg"))
                or os.path.exists(os.path.join(cvs_parent, ".hg_archival.txt"))
                or os.path.exists(os.path.join(cvs_parent, ".git_archival.txt"))
            ):
                package_path_in_target = os.path.join(
                    os.path.basename(cvs_parent), package_path_in_target
                )
                if os.path.dirname(cvs_parent) == "":
                    # stop at one level in the super project
                    break
                cvs_parent = os.path.dirname(cvs_parent)
            copy_paths.add(cvs_parent)
            requirements.append(
                os.path.join(target, _target_path(cvs_parent), package_path_in_target)
            )
    for copy_path in copy_paths:
        src, dest = copy_path + os.path.sep, os.path.join(
            target, _target_path(copy_path)
        )
        _logger.info("Copying package %s to %s", src, dest)
        if not os.path.exists(target):
            os.mkdir(target)
        cmd = [
            "rsync",
            "--delete",
            "--include=core",
            "--copy-links",
            "--exclude='*.pyc'",
            "-r",
            "--times",
            src,
            dest,
        ]
        _logger.debug(" ".join(cmd))
        check_call(cmd)
    # The option to ignore the python version can be an issue with newer syntax.
    if packages_to_compile:
        # try with build if present, fallback to pip wheel otherwise
        try:
            # build is not usable directly so use a subprocess.
            # Disable flake8, this is just to test that the package is installed.
            import build  # type: ignore[import] # noqa: F401

            cmd = [
                sys.executable,
                "-m",
                "build",
                "--outdir",
                target,
            ] + packages_to_compile
            _logger.debug(" ".join(cmd))
            check_call(cmd)
        except ImportError:
            # pip is not usable directly, it is indicated in its doc to use subprocess
            # instead.
            cmd = [
                sys.executable,
                "-m",
                "pip",
                "wheel",
                "--no-deps",
                "-w",
                target,
                "--ignore-requires-python",
            ] + packages_to_compile
        _logger.debug(" ".join(cmd))
        check_call(cmd)

        try:
            # Disable flake8, this is just to test that the package is installed
            import twine  # type: ignore[import] # noqa: F401

            cmd = [sys.executable, "-m", "twine", "check", f"{target}/*"]
            check_call(cmd)
        except ImportError:
            _logger.warn("No twine, no check done")

    for entry in os.listdir(target):
        if entry.endswith(".whl"):
            requirements.append(os.path.join(".", target, entry))
    # Write requirements
    # pip needs to be run with pip install -r <target>/requirements for the path in the
    # file to be valid
    with open(os.path.join(target, "requirements"), "wt") as f:
        f.write("\n".join(requirements))

    # copy setup files to odoo_setup
    source = "setup/"
    target = "odoo_setup"
    if os.path.exists(source):
        _logger.info("Copying Odoo %s files to %s", source, target)
        if not os.path.exists(target):
            os.mkdir(target)
        cmd = [
            "rsync",
            "--delete",
            "--cvs-exclude",
            "--include=core",
            "--exclude=.hg*",
            "--exclude=.git*",
            "--copy-links",
            "--exclude='*.pyc'",
            "-r",
            "--times",
        ] + [source, target]
        _logger.debug(" ".join(cmd))
        check_call(cmd)

    list_modules()


if __name__ == "__main__":
    sys.exit(main())
