#!/usr/bin/env python3
"""Run prettier in a docker
"""
import logging
import os
import sys
from typing import List, Optional

from .config import ODOO_7, ODOO_8, ODOO_9, ODOO_10, Config
from .docker_build import add_build_options, build_local_image, get_build_options
from .docker_client import DockerClient, get_volumes
from .parsing import apply, basic_parser

__version__ = "1.1.1"
__date__ = "2022-02-17"
__updated__ = "2022-10-03"

_logger = logging.getLogger(__name__)

_PRETTIER_DEST = "prettier"
"""Name of destination variable used for prettier in parsing"""


def __parser():
    program_version = __version__
    program_build_date = str(__updated__)
    program_version_message = "%%(prog)s %s (%s)" % (
        program_version,
        program_build_date,
    )
    program_shortdesc = __doc__.split(".", maxsplit=1)[0]
    program_license = f"""{program_shortdesc}

      Created by Vincent Hatakeyama on {__date__}.
      Copyright 2022 XCG Consulting. All rights reserved.

      Licensed under the MIT License

      Distributed on an "AS IS" basis without warranties
      or conditions of any kind, either express or implied.

    USAGE
    """
    parser = basic_parser(program_license, program_version_message)

    parser.add_argument(
        help="Run pylint on given directory (relative path, for example "
        "github.com/OCA/server-auth)",
        dest=_PRETTIER_DEST,
    )
    parser.add_argument(
        "--write",
        help="Write changes",
        action="store_true",
    )

    return parser


def main(argv: Optional[List[str]] = None) -> int:
    """Copy modules for a build, callable version that parses arguments"""
    parser = __parser()
    add_build_options(parser)
    nmspc = parser.parse_args(argv)
    apply(nmspc)
    return prettier(
        getattr(nmspc, _PRETTIER_DEST), nmspc.write, get_build_options(nmspc)
    )


def prettier(directory: str, write: bool, build_options):
    """Run prettier"""
    config = Config()

    if config.odoo_type in (ODOO_7, ODOO_8, ODOO_9, ODOO_10):
        image = "quay.orus.io/xcgd/prettier"
        tag = "2.5.1"
    else:
        # Only works on Odoo 11+ where prettier is in the image
        build_local_image(build_options)
        image = config.local_image
        tag = config.local_tag

    command = ["."]
    if write:
        command.append("--write")

    pwd = os.environ["PWD"]
    volumes, path = get_volumes(pwd, directory, write)

    _logger.info("Running prettier in %s", path)
    return DockerClient.run(
        image,
        tag,
        {
            "entrypoint": "prettier",
            "user": os.getuid(),
            "command": command,
            "volumes": volumes,
            "working_dir": path,
            "tty": True,
        },
        pull=False,
    )


if __name__ == "__main__":
    sys.exit(main())
